@@ -1,17 +1,29 @@
 # exporting haiway symbols for easier usage
 from haiway import (
+    META_EMPTY,
     MISSING,
     AsyncQueue,
     AsyncStream,
     AttributePath,
     AttributeRequirement,
+    ContextPreset,
     Default,
     DefaultValue,
     Disposable,
     Disposables,
     File,
     FileAccess,
+    HTTPClient,
+    HTTPClientError,
+    HTTPHeaders,
+    HTTPQueryParams,
+    HTTPResponse,
+    HTTPStatusCode,
     LoggerObservability,
+    Meta,
+    MetaTags,
+    MetaValue,
+    MetaValues,
     Missing,
     MissingContext,
     MissingState,
@@ -49,7 +61,6 @@ from haiway import (
     stream_concurrently,
     throttle,
     timeout,
-    traced,
     unwrap_missing,
     without_missing,
 )
@@ -73,13 +84,6 @@ from draive.choice import (
     SelectionException,
     choice_completion,
 )
-from draive.commons import (
-    META_EMPTY,
-    Meta,
-    MetaTags,
-    MetaValue,
-    MetaValues,
-)
 from draive.configuration import Config, Configuration
 from draive.conversation import (
     Conversation,
@@ -212,8 +216,8 @@ from draive.tokenization import Tokenization
 from draive.tools import (
     Tool,
     Toolbox,
+    ToolError,
     ToolErrorFormatting,
-    ToolException,
     ToolHandling,
     Tools,
     tool,
@@ -254,6 +258,7 @@ __all__ = (
     "ChoiceOption",
     "Config",
     "Configuration",
+    "ContextPreset",
     "Conversation",
     "ConversationElement",
     "ConversationEvent",
@@ -278,6 +283,12 @@ __all__ = (
     "GuardrailsOutputModerationException",
     "GuardrailsQualityException",
     "GuardrailsQualityVerification",
+    "HTTPClient",
+    "HTTPClientError",
+    "HTTPHeaders",
+    "HTTPQueryParams",
+    "HTTPResponse",
+    "HTTPStatusCode",
     "ImageEmbedding",
     "ImageGeneration",
     "Instruction",
@@ -374,8 +385,8 @@ __all__ = (
     "TextGeneration",
     "Tokenization",
     "Tool",
+    "ToolError",
     "ToolErrorFormatting",
-    "ToolException",
     "ToolHandling",
     "Toolbox",
     "Tools",
@@ -417,7 +428,6 @@ __all__ = (
     "throttle",
     "timeout",
     "tool",
-    "traced",
     "unwrap_missing",
     "vector_similarity_score",
     "vector_similarity_search",
