@@ -106,7 +106,7 @@ class GeminiLMMGeneration(GeminiAPI):
         tools = tools or LMMTools.none
         generation_config: GeminiGenerationConfig = config or ctx.state(GeminiGenerationConfig)
 
-        with ctx.scope("gemini_lmm_completion", generation_config):
+        async with ctx.scope("gemini_lmm_completion", generation_config):
             request_config: GenerateContentConfigDict = _prepare_request_config(
                 instruction=instruction,
                 context=context,
@@ -114,9 +114,9 @@ class GeminiLMMGeneration(GeminiAPI):
                 output=output,
                 config=generation_config,
             )
-            request_content: ContentListUnionDict = list(
-                chain.from_iterable([context_element_as_content(element) for element in context])
-            )
+            request_content: ContentListUnionDict = [
+                context_element_as_content(element) for element in context
+            ]
             output_decoder: LMMOutputDecoder = lmm_output_decoder(output)
 
             if stream:
@@ -453,6 +453,7 @@ def _record_usage_metrics(
         metric="lmm.input_tokens",
         value=usage.prompt_token_count or 0,
         unit="tokens",
+        kind="counter",
         attributes={"lmm.model": model},
     )
     ctx.record(
@@ -460,6 +461,7 @@ def _record_usage_metrics(
         metric="lmm.input_tokens.cached",
         value=usage.cached_content_token_count or 0,
         unit="tokens",
+        kind="counter",
         attributes={"lmm.model": model},
     )
     ctx.record(
@@ -467,6 +469,7 @@ def _record_usage_metrics(
         metric="lmm.output_tokens",
         value=usage.candidates_token_count or 0,
         unit="tokens",
+        kind="counter",
         attributes={"lmm.model": model},
     )
 
