@@ -1,6 +1,7 @@
 from typing import Any, Protocol, runtime_checkable
 
-from draive.commons import Meta, MetaValues
+from haiway import Meta, MetaValues
+
 from draive.multimodal import MultimodalContent
 
 __all__ = (
