@@ -1,10 +1,11 @@
 from collections.abc import Mapping, Sequence
 from typing import Any, final
 
+from haiway import ctx
+
 from draive.instructions.types import (
     Instruction,
     InstructionDeclaration,
-    InstructionMissing,
 )
 
 __all__ = ("InstructionsVolatileStorage",)
@@ -12,6 +13,11 @@ __all__ = ("InstructionsVolatileStorage",)
 
 @final
 class InstructionsVolatileStorage:
+    __slots__ = (
+        "_listing",
+        "_storage",
+    )
+
     def __init__(
         self,
         instructions: Sequence[Instruction],
@@ -23,19 +29,19 @@ class InstructionsVolatileStorage:
             instruction.declaration for instruction in instructions
         ]
 
-    async def listing(
+    async def fetch_list(
         self,
         **extra: Any,
     ) -> Sequence[InstructionDeclaration]:
         return self._listing
 
-    async def instruction(
+    async def fetch_instruction(
         self,
         name: str,
         *,
         arguments: Mapping[str, str | float | int] | None = None,
         **extra: Any,
-    ) -> Instruction:
+    ) -> Instruction | None:
         if instruction := self._storage.get(name):
             if arguments:
                 return instruction.updated(arguments={**instruction.arguments, **arguments})
@@ -44,4 +50,5 @@ class InstructionsVolatileStorage:
                 return instruction
 
         else:
-            raise InstructionMissing(f"Instruction '{name}' is not defined")
+            ctx.log_debug(f"Instruction '{name}' is not defined")
+            return None
