@@ -40,7 +40,7 @@ from draive.mistral.api import MistralAPI
 from draive.mistral.config import MistralChatConfig
 from draive.mistral.lmm import (
     content_chunk_as_content_element,
-    content_element_as_content_chunk,
+    content_chunks,
     context_element_as_messages,
     output_as_response_declaration,
     tools_as_tool_config,
@@ -98,7 +98,7 @@ class MistralLMMGeneration(MistralAPI):
     ) -> AsyncIterator[LMMStreamOutput] | LMMOutput:
         completion_config: MistralChatConfig = config or ctx.state(MistralChatConfig)
         tools = tools or LMMTools.none
-        with ctx.scope("mistral_lmm_completion", completion_config):
+        async with ctx.scope("mistral_lmm_completion", completion_config):
             ctx.record(
                 ObservabilityLevel.INFO,
                 attributes={
@@ -124,10 +124,7 @@ class MistralLMMGeneration(MistralAPI):
                 messages.append(
                     {
                         "role": "assistant",
-                        "content": [
-                            content_element_as_content_chunk(element)
-                            for element in MultimodalContent.of(prefill).parts
-                        ],
+                        "content": list(content_chunks(MultimodalContent.of(prefill).parts)),
                         "prefix": True,
                     }
                 )
@@ -210,15 +207,17 @@ class MistralLMMGeneration(MistralAPI):
             ctx.record(
                 ObservabilityLevel.INFO,
                 metric="lmm.input_tokens",
-                value=usage.prompt_tokens,
+                value=usage.prompt_tokens or 0,
                 unit="tokens",
+                kind="counter",
                 attributes={"lmm.model": completion.model},
             )
             ctx.record(
                 ObservabilityLevel.INFO,
                 metric="lmm.output_tokens",
-                value=usage.completion_tokens,
+                value=usage.completion_tokens or 0,
                 unit="tokens",
+                kind="counter",
                 attributes={"lmm.model": completion.model},
             )
 
@@ -264,9 +263,8 @@ class MistralLMMGeneration(MistralAPI):
             lmm_completion = None
 
         if tool_calls := completion_message.tool_calls:
-            completion_tool_calls = LMMToolRequests(
-                content=lmm_completion.content if lmm_completion else None,
-                requests=[
+            completion_tool_calls = LMMToolRequests.of(
+                [
                     LMMToolRequest(
                         identifier=call.id or uuid4().hex,
                         tool=call.function.name,
@@ -276,6 +274,7 @@ class MistralLMMGeneration(MistralAPI):
                     )
                     for call in tool_calls
                 ],
+                content=lmm_completion.content if lmm_completion else None,
             )
             ctx.record(
                 ObservabilityLevel.INFO,
@@ -330,15 +329,17 @@ class MistralLMMGeneration(MistralAPI):
                         ctx.record(
                             ObservabilityLevel.INFO,
                             metric="lmm.input_tokens",
-                            value=usage.prompt_tokens,
+                            value=usage.prompt_tokens or 0,
                             unit="tokens",
+                            kind="counter",
                             attributes={"lmm.model": completion_chunk.data.model},
                         )
                         ctx.record(
                             ObservabilityLevel.INFO,
                             metric="lmm.output_tokens",
-                            value=usage.completion_tokens,
+                            value=usage.completion_tokens or 0,
                             unit="tokens",
+                            kind="counter",
                             attributes={"lmm.model": completion_chunk.data.model},
                         )
 
