@@ -88,7 +88,7 @@ class OpenAILMMGeneration(OpenAIAPI):
     ) -> AsyncIterator[LMMStreamOutput] | LMMOutput:
         tools = tools or LMMTools.none
         completion_config: OpenAIChatConfig = config or ctx.state(OpenAIChatConfig).updated(**extra)
-        with ctx.scope("openai_lmm_completion", completion_config):
+        async with ctx.scope("openai_lmm_completion", completion_config):
             ctx.record(
                 ObservabilityLevel.INFO,
                 attributes={
@@ -242,6 +242,7 @@ class OpenAILMMGeneration(OpenAIAPI):
                 metric="lmm.input_tokens",
                 value=usage.prompt_tokens,
                 unit="tokens",
+                kind="counter",
                 attributes={"lmm.model": completion.model},
             )
             ctx.record(
@@ -249,6 +250,7 @@ class OpenAILMMGeneration(OpenAIAPI):
                 metric="lmm.output_tokens",
                 value=usage.completion_tokens,
                 unit="tokens",
+                kind="counter",
                 attributes={"lmm.model": completion.model},
             )
 
@@ -287,9 +289,8 @@ class OpenAILMMGeneration(OpenAIAPI):
 
         if tool_calls := completion_message.tool_calls:
             assert tools, "Requesting tool call without tools"  # nosec: B101
-            completion_tool_calls = LMMToolRequests(
-                content=lmm_completion.content if lmm_completion else None,
-                requests=[
+            completion_tool_calls = LMMToolRequests.of(
+                [
                     LMMToolRequest(
                         identifier=call.id,
                         tool=call.function.name,
@@ -299,6 +300,7 @@ class OpenAILMMGeneration(OpenAIAPI):
                     )
                     for call in tool_calls
                 ],
+                content=lmm_completion.content if lmm_completion else None,
             )
             ctx.record(
                 ObservabilityLevel.INFO,
@@ -387,6 +389,7 @@ class OpenAILMMGeneration(OpenAIAPI):
                         metric="lmm.input_tokens",
                         value=usage.prompt_tokens,
                         unit="tokens",
+                        kind="counter",
                         attributes={"lmm.model": part.model},
                     )
                     ctx.record(
@@ -394,6 +397,7 @@ class OpenAILMMGeneration(OpenAIAPI):
                         metric="lmm.output_tokens",
                         value=usage.completion_tokens,
                         unit="tokens",
+                        kind="counter",
                         attributes={"lmm.model": part.model},
                     )
 
