@@ -12,14 +12,23 @@ from collections.abc import (
 from typing import Any, ClassVar, Literal, Protocol, Self, cast, final, overload
 from uuid import uuid4
 
-from haiway import Disposable, Disposables, State, cache, ctx, retry
+from haiway import (
+    Disposable,
+    Disposables,
+    Meta,
+    MetaValue,
+    MetaValues,
+    State,
+    cache,
+    ctx,
+    retry,
+)
 
-from draive.commons import Meta, MetaValue, MetaValues
 from draive.evaluation import (
     EvaluatorResult,
+    EvaluatorScenarioResult,
     PreparedEvaluator,
-    PreparedScenarioEvaluator,
-    ScenarioEvaluatorResult,
+    PreparedEvaluatorScenario,
 )
 from draive.instructions import Instruction
 from draive.lmm import (
@@ -205,7 +214,7 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.predefined"):
+            async with ctx.scope("stage.predefined"):
                 return state.updated(
                     context=(*state.context, *context_extension),
                     result=completion_result if completion_result is not None else state.result,
@@ -256,7 +265,7 @@ class Stage:
                     *,
                     state: StageState,
                 ) -> StageState:
-                    with ctx.scope("stage.memory_recall"):
+                    async with ctx.scope("stage.memory_recall"):
                         return await memory.recall()
 
             case "merge":
@@ -265,7 +274,7 @@ class Stage:
                     *,
                     state: StageState,
                 ) -> StageState:
-                    with ctx.scope("stage.memory_recall"):
+                    async with ctx.scope("stage.memory_recall"):
                         return state.merged(await memory.recall())
 
         return cls(
@@ -304,7 +313,7 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.memory_remember"):
+            async with ctx.scope("stage.memory_remember"):
                 await memory.remember(state)
                 return state
 
@@ -382,7 +391,7 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.completion"):
+            async with ctx.scope("stage.completion"):
                 context, result = await _lmm_completion(
                     instruction=instruction,
                     context=(*state.context, *context_extension),
@@ -467,7 +476,7 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.completion.prompting"):
+            async with ctx.scope("stage.completion.prompting"):
                 context, result = await _lmm_completion(
                     instruction=instruction,
                     context=(
@@ -534,7 +543,7 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.completion.loopback"):
+            async with ctx.scope("stage.completion.loopback"):
                 if not state.context or not isinstance(state.context[-1], LMMCompletion):
                     ctx.log_warning("loopback_completion has been skipped due to invalid context")
                     return state
@@ -570,6 +579,72 @@ class Stage:
             meta=Meta.of(meta),
         )
 
+    @classmethod
+    def result_completion(
+        cls,
+        *,
+        instruction: Instruction | str | None = None,
+        tools: Toolbox | Iterable[Tool] | None = None,
+        output: LMMOutputSelection = "auto",
+        meta: Meta | MetaValues | None = None,
+        **extra: Any,
+    ) -> Self:
+        """
+        Creates a Stage that takes the current result and uses it as the new input.
+
+        This Stage takes the current stage result, appends it as a new input, and executes
+        a new completion using the updated context.
+
+        Parameters
+        ----------
+        instruction : Instruction | str | None
+            Optional instruction or guidance for the LMM.
+        tools : Toolbox | Iterable[Tool] | None
+            Optional tools that the LMM can use during completion generation.
+        output : LMMOutputSelection
+            Controls the modality/type of the generated completion, default is "auto".
+        meta: Meta | MetaValues | None = None
+            Additional stage metadata including tags, description etc.
+        **extra : Any
+            Additional parameters to pass to the LMM invocation.
+
+        Returns
+        -------
+        Self
+            A new Stage instance that performs the result completion.
+
+        Raises
+        ------
+        RuntimeError
+            If the LMM exceeds the limit of recursive tool calls.
+        """
+        toolbox: Toolbox = Toolbox.of(tools)
+
+        async def stage(
+            *,
+            state: StageState,
+        ) -> StageState:
+            async with ctx.scope("stage.completion.result"):
+                context, result = await _lmm_completion(
+                    instruction=instruction,
+                    context=(
+                        *state.context,
+                        LMMInput.of(state.result),
+                    ),
+                    toolbox=toolbox,
+                    output=output,
+                    **extra,
+                )
+                return state.updated(
+                    context=context,
+                    result=result,
+                )
+
+        return cls(
+            stage,
+            meta=Meta.of(meta),
+        )
+
     @classmethod
     def transform_result(
         cls,
@@ -601,7 +676,7 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.transform.result"):
+            async with ctx.scope("stage.transform.result"):
                 return state.updated(result=await transformation(state.result))
 
         return cls(
@@ -640,7 +715,7 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.transform.context"):
+            async with ctx.scope("stage.transform.context"):
                 return state.updated(context=await transformation(state.context))
 
         return cls(
@@ -786,46 +861,79 @@ class Stage:
         >>> stage = Stage.tool_call(get_weather, location="New York")
         """
         assert not args, "Positional arguments are not supported"  # nosec: B101
-        direct_result: bool
-        match tool.handling:
-            case "auto":
-                direct_result = False
-
-            case "direct":
-                direct_result = True
 
         async def stage(
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.tool_call"):
+            async with ctx.scope("stage.tool_call"):
+                request_id: str = uuid4().hex
                 # Create tool request representing the call
                 tool_request: LMMToolRequest = LMMToolRequest.of(
-                    uuid4().hex,
+                    request_id,
                     tool=tool.name,
                     arguments=kwargs,
                 )
 
-                tool_response: LMMToolResponse
                 try:
-                    # Execute the tool directly
-                    result = await tool(*args, **kwargs)
+                    tool_response: LMMToolResponse
+                    result: MultimodalContent
+                    match tool.handling:
+                        case "auto":
+                            tool_response = LMMToolResponse.of(
+                                tool_request.identifier,
+                                tool=tool.name,
+                                content=MultimodalContent.of(
+                                    # Execute the tool directly
+                                    tool.format_result(await tool(*args, **kwargs))
+                                ),
+                                handling="result",
+                            )
+                            result = state.result  # result unchanged
 
-                    # Create tool response with the result
-                    tool_response = LMMToolResponse.of(
-                        tool_request.identifier,
-                        tool=tool.name,
-                        content=MultimodalContent.of(tool.format_result(result)),
-                        handling="direct_result" if direct_result else "result",
-                    )
+                        case "direct":
+                            tool_result: MultimodalContent = tool.format_result(
+                                await tool(*args, **kwargs)
+                            )
+                            tool_response = LMMToolResponse.of(
+                                tool_request.identifier,
+                                tool=tool.name,
+                                content=tool_result,
+                                handling="completion",
+                            )
+                            result = tool_result  # result is the tool result
+
+                        case "extend":
+                            tool_result: MultimodalContent = tool.format_result(
+                                await tool(*args, **kwargs)
+                            )
+                            tool_response = LMMToolResponse.of(
+                                tool_request.identifier,
+                                tool=tool.name,
+                                content=tool_result,
+                                handling="extension",
+                            )
+                            result = state.result.appending(tool_result)  # result extended
+
+                        case "spawn":
+                            ctx.spawn(tool, *args, **kwargs)
+                            tool_response = LMMToolResponse.of(
+                                tool_request.identifier,
+                                tool=tool.name,
+                                # placeholder for detached tool call
+                                content=MultimodalContent.of(f"{tool.name} tool has been called"),
+                                handling="detached",
+                            )
+                            result = state.result  # result unchanged
 
                 except Exception as exc:
                     tool_response = LMMToolResponse.of(
                         tool_request.identifier,
                         tool=tool.name,
-                        content=MultimodalContent.of(tool.format_failure(exc)),
-                        handling="direct_result" if direct_result else "error",
+                        content=MultimodalContent.of(tool.format_error(exc)),
+                        handling="error",
                     )
+                    result = state.result  # result unchanged
 
                 return state.updated(
                     context=(
@@ -833,7 +941,7 @@ class Stage:
                         LMMToolRequests.of((tool_request,)),
                         LMMToolResponses.of((tool_response,)),
                     ),
-                    result=tool_response.content if direct_result else state.result,
+                    result=result,
                 )
 
         return cls(
@@ -844,7 +952,7 @@ class Stage:
     @classmethod
     def result_evaluation(
         cls,
-        evaluator: PreparedScenarioEvaluator[MultimodalContent]
+        evaluator: PreparedEvaluatorScenario[MultimodalContent]
         | PreparedEvaluator[MultimodalContent],
         /,
         *,
@@ -858,7 +966,7 @@ class Stage:
 
         Parameters
         ----------
-        evaluator : PreparedScenarioEvaluator[MultimodalContent]
+        evaluator : PreparedEvaluatorScenario[MultimodalContent]
         | PreparedEvaluator[MultimodalContent]
             The evaluator or scenario evaluator to use for evaluation.
         meta: Meta | MetaValues | None = None
@@ -878,21 +986,21 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.result_evaluation"):
-                evaluation_result: ScenarioEvaluatorResult | EvaluatorResult = await evaluator(
+            async with ctx.scope("stage.result_evaluation"):
+                evaluation_result: EvaluatorScenarioResult | EvaluatorResult = await evaluator(
                     state.result
                 )
 
                 if evaluation_result.passed:
                     return state  # evaluation passed, keep going
 
-                score: float = evaluation_result.relative_score
-                report: str = evaluation_result.report(include_details=__debug__)
+                performance: float = evaluation_result.performance
+                report: str = evaluation_result.report(detailed=__debug__)
                 raise StageException(
-                    f"Result evaluation failed with relative score: {score * 100:.2f}%",
+                    f"Result evaluation failed with relative score: {performance:.2f}%",
                     state=state,
                     meta={
-                        "evaluation_score": score,
+                        "evaluation_performance": performance,
                         "evaluation_report": report,
                     },
                 )
@@ -905,7 +1013,7 @@ class Stage:
     @classmethod
     def context_evaluation(
         cls,
-        evaluator: PreparedScenarioEvaluator[LMMContext] | PreparedEvaluator[LMMContext],
+        evaluator: PreparedEvaluatorScenario[LMMContext] | PreparedEvaluator[LMMContext],
         /,
         *,
         meta: Meta | MetaValues | None = None,
@@ -918,7 +1026,7 @@ class Stage:
 
         Parameters
         ----------
-        evaluator : PreparedScenarioEvaluator[Value] | PreparedEvaluator[Value]
+        evaluator : PreparedEvaluatorScenario[Value] | PreparedEvaluator[Value]
             The evaluator or scenario evaluator to use for evaluation.
         meta: Meta | MetaValues | None = None
             Additional stage metadata including tags, description etc.
@@ -937,21 +1045,21 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.context_evaluation"):
-                evaluation_result: ScenarioEvaluatorResult | EvaluatorResult = await evaluator(
+            async with ctx.scope("stage.context_evaluation"):
+                evaluation_result: EvaluatorScenarioResult | EvaluatorResult = await evaluator(
                     state.context
                 )
 
                 if evaluation_result.passed:
                     return state  # evaluation passed, keep going
 
-                score: float = evaluation_result.relative_score
-                report: str = evaluation_result.report(include_details=__debug__)
+                performance: float = evaluation_result.performance
+                report: str = evaluation_result.report(detailed=__debug__)
                 raise StageException(
-                    f"Context evaluation failed with relative score: {score * 100:.2f}%",
+                    f"Context evaluation failed with relative score: {performance:.2f}%",
                     state=state,
                     meta={
-                        "evaluation_score": score,
+                        "evaluation_performance": performance,
                         "evaluation_report": report,
                     },
                 )
@@ -1012,12 +1120,12 @@ class Stage:
                     *,
                     state: StageState,
                 ) -> StageState:
-                    with ctx.scope("stage.loop"):
+                    async with ctx.scope("stage.loop"):
                         current_state: StageState = state
                         iteration: int = 0
 
                         while await condition(state=current_state, iteration=iteration):
-                            with ctx.scope("stage.loop.iteration"):
+                            async with ctx.scope("stage.loop.iteration"):
                                 for execution in stage_executions:
                                     current_state = await execution(state=current_state)
 
@@ -1031,12 +1139,12 @@ class Stage:
                     *,
                     state: StageState,
                 ) -> StageState:
-                    with ctx.scope("stage.loop"):
+                    async with ctx.scope("stage.loop"):
                         current_state: StageState = state
                         iteration: int = 0
 
                         while True:
-                            with ctx.scope("stage.loop.iteration"):
+                            async with ctx.scope("stage.loop.iteration"):
                                 for stage_execution in stage_executions:
                                     current_state = await stage_execution(state=current_state)
 
@@ -1136,7 +1244,7 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.concurrent"):
+            async with ctx.scope("stage.concurrent"):
                 return await merge(
                     branches=cast(  # we are converting errors witin __call__
                         Sequence[StageState | StageException],
@@ -1246,7 +1354,7 @@ class Stage:
             *,
             state: StageState,
         ) -> StageState:
-            with ctx.scope("stage.router"):
+            async with ctx.scope("stage.router"):
                 selection: str = await routing(
                     state=state,
                     options=options,
@@ -1414,8 +1522,23 @@ class Stage:
                     *,
                     state: StageState,
                 ) -> StageState:
-                    async with ctx_disposables:
-                        return await execution(state=state)
+                    result_state: StageState
+                    try:
+                        with ctx.updated(*await ctx_disposables.prepare()):
+                            result_state = await execution(state=state)
+
+                    except BaseException as exc:
+                        await ctx_disposables.dispose(
+                            exc_type=type(exc),
+                            exc_val=exc,
+                            exc_tb=exc.__traceback__,
+                        )
+                        raise exc
+
+                    else:
+                        await ctx_disposables.dispose()
+
+                    return result_state
 
             case (ctx_state, None):
 
@@ -1618,7 +1741,7 @@ class Stage:
                 return await execution(state=state)
 
             except Exception as exc:
-                with ctx.scope("stage.fallback"):
+                async with ctx.scope("stage.fallback"):
                     if exc in exceptions:
                         return await fallback_execution(state=state)
 
@@ -1875,7 +1998,7 @@ class Stage:
         MultimodalContent
             The result produced by executing the Stage.
         """
-        with ctx.scope("stage.execution"):
+        async with ctx.scope("stage.execution"):
             initial_context: LMMContext
             match context:
                 case None:
@@ -1906,47 +2029,61 @@ async def _lmm_completion(
 ) -> tuple[LMMContext, MultimodalContent]:
     current_context: LMMContext = context
     formatted_instruction: LMMInstruction | None = Instruction.formatted(instruction)
-    repetition_level: int = 0
+    tools_turn: int = 0
+    result: LMMCompletion
+    result_extension: MultimodalContent = MultimodalContent.empty
     while True:
+        ctx.log_debug("...requesting completion...")
         match await LMM.completion(
             instruction=formatted_instruction,
             context=current_context,
             output=output,
-            tools=toolbox.available_tools(repetition_level=repetition_level),
+            tools=toolbox.available_tools(tools_turn=tools_turn),
             **extra,
         ):
             case LMMCompletion() as completion:
-                return (
-                    (*current_context, completion),
-                    completion.content,
-                )
-
-            case LMMToolRequests() as tool_requests:
-                tool_responses: LMMToolResponses = await toolbox.respond_all(tool_requests)
-
-                if direct_results := [
-                    response.content
-                    for response in tool_responses.responses
-                    if response.handling == "direct_result"
-                ]:
-                    direct_content: MultimodalContent = MultimodalContent.of(*direct_results)
-
-                    return (
-                        (
-                            *current_context,
-                            LMMCompletion.of(direct_content),
-                        ),
-                        direct_content,
+                ctx.log_debug("...received result...")
+                if result_extension:
+                    result = completion.updated(
+                        content=result_extension.appending(completion.content)
                     )
 
                 else:
-                    current_context = (
-                        *current_context,
-                        tool_requests,
-                        tool_responses,
-                    )
+                    result = completion
 
-        repetition_level += 1  # continue tracking repetetive tool calls
+                break  # proceed to resolving
+
+            case LMMToolRequests() as tool_requests:
+                ctx.log_debug(f"...received tool requests (turn {tools_turn})...")
+                # skip tool_requests.content - no need for extra comments
+                tool_responses: LMMToolResponses = await toolbox.respond_all(tool_requests)
+
+                if completion := tool_responses.completion(extension=result_extension):
+                    ctx.log_debug("...received tools direct result...")
+                    result = completion
+                    break  # proceed to resolving
+
+                elif extension := tool_responses.completion_extension():
+                    ctx.log_debug("...received tools result extension...")
+                    result_extension = result_extension.appending(extension)
+
+                ctx.log_debug("...received tools responses...")
+                current_context = (
+                    *current_context,
+                    tool_requests,
+                    tool_responses,
+                )
+
+        tools_turn += 1  # continue with next turn
+
+    ctx.log_debug("...finalized result!")
+    return (
+        (
+            *current_context,
+            result,
+        ),
+        result.content,
+    )
 
 
 async def _lmm_routing(
