@@ -4,8 +4,8 @@ from draive.tools.toolbox import Toolbox
 from draive.tools.types import (
     Tool,
     ToolAvailabilityChecking,
+    ToolError,
     ToolErrorFormatting,
-    ToolException,
     ToolHandling,
     ToolResultFormatting,
     ToolsFetching,
@@ -15,8 +15,8 @@ __all__ = (
     "FunctionTool",
     "Tool",
     "ToolAvailabilityChecking",
+    "ToolError",
     "ToolErrorFormatting",
-    "ToolException",
     "ToolHandling",
     "ToolResultFormatting",
     "Toolbox",
