@@ -88,7 +88,7 @@ class VLLMLMMGeneration(VLLMAPI):
     ) -> AsyncIterator[LMMStreamOutput] | LMMOutput:
         completion_config: VLLMChatConfig = config or ctx.state(VLLMChatConfig)
         tools = tools or LMMTools.none
-        with ctx.scope("vllm_lmm_completion", completion_config):
+        async with ctx.scope("vllm_lmm_completion", completion_config):
             ctx.record(
                 ObservabilityLevel.INFO,
                 attributes={
@@ -112,7 +112,7 @@ class VLLMLMMGeneration(VLLMAPI):
                     [
                         context_element_as_messages(
                             element,
-                            config=completion_config,
+                            vision_details=completion_config.vision_details,
                         )
                         for element in context
                     ]
@@ -242,6 +242,7 @@ class VLLMLMMGeneration(VLLMAPI):
                 metric="lmm.input_tokens",
                 value=usage.prompt_tokens,
                 unit="tokens",
+                kind="counter",
                 attributes={"lmm.model": completion.model},
             )
             ctx.record(
@@ -249,6 +250,7 @@ class VLLMLMMGeneration(VLLMAPI):
                 metric="lmm.output_tokens",
                 value=usage.completion_tokens,
                 unit="tokens",
+                kind="counter",
                 attributes={"lmm.model": completion.model},
             )
 
@@ -284,9 +286,8 @@ class VLLMLMMGeneration(VLLMAPI):
 
         if tool_calls := completion_message.tool_calls:
             assert tools, "Requesting tool call without tools"  # nosec: B101
-            completion_tool_calls = LMMToolRequests(
-                content=lmm_completion.content if lmm_completion else None,
-                requests=[
+            completion_tool_calls = LMMToolRequests.of(
+                [
                     LMMToolRequest(
                         identifier=call.id,
                         tool=call.function.name,
@@ -296,6 +297,7 @@ class VLLMLMMGeneration(VLLMAPI):
                     )
                     for call in tool_calls
                 ],
+                content=lmm_completion.content if lmm_completion else None,
             )
             ctx.record(
                 ObservabilityLevel.INFO,
@@ -382,6 +384,7 @@ class VLLMLMMGeneration(VLLMAPI):
                         metric="lmm.input_tokens",
                         value=usage.prompt_tokens,
                         unit="tokens",
+                        kind="counter",
                         attributes={"lmm.model": part.model},
                     )
                     ctx.record(
@@ -389,6 +392,7 @@ class VLLMLMMGeneration(VLLMAPI):
                         metric="lmm.output_tokens",
                         value=usage.completion_tokens,
                         unit="tokens",
+                        kind="counter",
                         attributes={"lmm.model": part.model},
                     )
 
